@@ -25,33 +25,31 @@
 
 #include "langutils.h"
 #include "khangmanview.h"
-#include "sharedkvtmlfiles.h"
 
 #include <QApplication>
 #include <QBitmap>
 #include <QCheckBox>
-#include <QPainter>
+#include <QDebug>
 #include <QDir>
+#include <QFileDialog>
+#include <QIcon>
+#include <QPainter>
+#include <QStandardPaths>
+#include <QStatusBar>
 
 #include <KSelectAction>
 #include <KToggleAction>
 #include <KActionCollection>
 #include <KConfigDialog>
-#include <KDebug>
-#include <KFileDialog>
-#include <KLineEdit>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMessageBox>
 #include <KStandardAction>
-#include <KStandardDirs>
-#include <KStatusBar>
 #include <KToolBar>
-#include <KIcon>
-#include <KGlobal>
 #include <KSharedConfig>
 #include <KRecentFilesAction>
 
 #include <KNS3/DownloadDialog>
+#include <sharedkvtmlfiles.h>
 
 KHangMan::KHangMan()
         : KXmlGuiWindow(), m_currentLevel(-1),
@@ -59,30 +57,23 @@ KHangMan::KHangMan()
           m_recent(0)
 {
     setObjectName(QLatin1String("KHangMan"));
-    
+
     setCentralWidget(m_view);
     setLanguages();
     setupStatusbar();
 
     //load the standard set of themes
-    khm_factory.addTheme(KStandardDirs::locate("data", "khangman/pics/standardthemes.xml"));   //TODO move it to better place    
+    m_khmfactory.addTheme(QStandardPaths::locate(QStandardPaths::GenericDataLocation, "khangman/themes/standardthemes.xml"));
 
     setupActions();
 
     // Toolbar for special characters
-    specialCharToolbar = toolBar("specialCharToolBar");
-    addToolBar ( Qt::BottomToolBarArea, specialCharToolbar);
+    m_specialCharToolbar = toolBar("specialCharToolBar");
+    addToolBar ( Qt::BottomToolBarArea, m_specialCharToolbar);
 
     loadSettings();
     setAccent();
     loadLangToolBar();
-    loadLevels();
-
-    // set the theme
-    slotChangeMode(Prefs::mode());   
-    show();
-    // Start a new game.
-    m_view->newGame();
 }
 
 KHangMan::~KHangMan()
@@ -92,30 +83,33 @@ KHangMan::~KHangMan()
 void KHangMan::setupActions()
 {
     // Game->New
-    KAction *newAct = KStandardAction::openNew(this, SLOT(slotNewGame()),
+    QAction *newAct = KStandardAction::openNew(this, SLOT(slotNewGame()),
                                        actionCollection());
     newAct->setToolTip(i18n( "Play with a new word" ));
 
-    KAction* fileOpen = KStandardAction::open(this, SLOT(slotFileOpen()), actionCollection());
+    QAction * fileOpen = KStandardAction::open(this, SLOT(slotFileOpen()), actionCollection());
     fileOpen->setWhatsThis(i18n("Opens an existing vocabulary document"));
     fileOpen->setToolTip(fileOpen->whatsThis());
 
-    hintAct = new KToggleAction(i18n("&Show Hint"), this);
-    hintAct->setToolTip(i18n( "Show/Hide the hint to help guessing the word" ));
-    actionCollection()->addAction("show_hint", hintAct );
+    m_hintAct = new KToggleAction(i18n("&Show Hint"), this);
+    m_hintAct->setToolTip(i18n( "Show/Hide the hint to help guessing the word" ));
+    actionCollection()->addAction("show_hint", m_hintAct );
     //hintAct->setCheckedState(KGuiItem(i18n("&Hide Hint")));
-    hintAct->setIcon(KIcon("games-hint"));
-    hintAct->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_H));
-    hintAct->setEnabled( m_view->hintExists() );
-    connect(hintAct, SIGNAL(triggered(bool)), this, SLOT(slotSetHint(bool)));
+    m_hintAct->setIcon(QIcon::fromTheme("games-hint"));
+    //set default shortcut key combination for displaying hints
+    actionCollection()->setDefaultShortcut(m_hintAct, QKeySequence(Qt::CTRL+Qt::Key_H));
+    m_hintAct->setEnabled( m_view->hintExists() );
+    connect(m_hintAct, SIGNAL(triggered(bool)), this, SLOT(slotSetHint(bool)));
     // Game->Get Words in New Language
-    KAction *newStuffAct  = new KAction(i18n("&Get Words in New Language..."), this);
+    QAction *newStuffAct  = new QAction(i18n("&Get Words in New Language..."), this);
     actionCollection()->addAction("downloadnewstuff", newStuffAct );
-    newStuffAct->setIcon(KIcon("get-hot-new-stuff"));
-    newStuffAct->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_G));
+    newStuffAct->setIcon(QIcon::fromTheme("get-hot-new-stuff"));
+    //set default shortcut key combination for GetHotNewStuff
+    actionCollection()->setDefaultShortcut(newStuffAct, QKeySequence(Qt::CTRL+Qt::Key_G));
     connect(newStuffAct, SIGNAL(triggered(bool)), this, SLOT(slotDownloadNewStuff()));
 
-    KStandardAction::quit(this, SLOT(slotQuit()), actionCollection());
+    //connect the quit action with close()
+    KStandardAction::quit(this, SLOT(close()), actionCollection());
 
     m_levelAction  = new KSelectAction(i18n("&Category"), this);
     actionCollection()->addAction("combo_level", m_levelAction );
@@ -137,19 +131,19 @@ void KHangMan::setupActions()
     // Mode.
     m_modeAction  = new KSelectAction(i18n("L&ook"), this);
     actionCollection()->addAction("combo_mode", m_modeAction );
-    connect(m_modeAction, SIGNAL(triggered(int)), this, SLOT(slotChangeMode(int)));
+    connect(m_modeAction, SIGNAL(triggered(int)), this, SLOT(slotChangeTheme(int)));
     //m_modeAction->setItems(KHMThemeFactory::instance()->themeList());
-    m_modeAction->setItems(khm_factory.themeList());
+    m_modeAction->setItems(m_khmfactory.themeList());
     m_modeAction->setCurrentItem(Prefs::mode());
     m_modeAction->setToolTip(i18n( "Choose the look and feel" ));
     m_modeAction->setWhatsThis(i18n( "Choose the look and feel" ));
 
-    config = KGlobal::config();
-    
-    m_recent=KStandardAction::openRecent(this, SLOT(slotOpenRecent(KUrl)), this);
-    m_recent->setWhatsThis(i18n("You can open last opened files")); //TODO: Check the description
+    m_config = KSharedConfig::openConfig();
+
+    m_recent=KStandardAction::openRecent(this, SLOT(slotOpenRecent(QUrl)), this);
+    m_recent->setWhatsThis(i18n("Quick access to some of the files that you have recently opened."));
     actionCollection()->addAction(m_recent->objectName(), m_recent);
-    m_recent->loadEntries(KConfigGroup(config, "KHangManRecent"));
+    m_recent->loadEntries(KConfigGroup(m_config, "KHangManRecent"));
     setupGUI();
 }
 
@@ -157,16 +151,18 @@ void KHangMan::setupActions()
 void KHangMan::setupStatusbar()
 {
     // set up the status bar
-    statusBar()->insertPermanentItem("   ", IDS_LEVEL,   0);
-    statusBar()->insertPermanentItem("   ", IDS_ACCENTS, 0);
-    statusBar()->insertItem("   ", IDS_WINS,    1);
-    statusBar()->insertItem("   ", IDS_LOSSES,  1);
-}
-
-
-void KHangMan::changeStatusbar(const QString& text, int id)
-{
-    statusBar()->changeItem(text, id);
+    m_levelLabel = new QLabel(this);
+    statusBar()->addPermanentWidget(m_levelLabel);
+    m_accentsLabel = new QLabel(this);
+    statusBar()->addPermanentWidget(m_accentsLabel);
+    m_winsLabel = new QLabel(this);
+    statusBar()->addPermanentWidget(m_winsLabel);
+    m_lossesLabel = new QLabel(this);
+    statusBar()->addPermanentWidget(m_lossesLabel);
+    //statusBar()->insertPermanentItem("   ", IDS_LEVEL,   0);
+    //statusBar()->insertPermanentItem("   ", IDS_ACCENTS, 0);
+    //statusBar()->insertItem("   ", IDS_WINS,    1);
+    //statusBar()->insertItem("   ", IDS_LOSSES,  1);
 }
 
 
@@ -174,21 +170,21 @@ void KHangMan::changeStatusbar(const QString& text, int id)
 //                               Slots
 
 
-void KHangMan::slotQuit()   //TODO: Isn't called when "X" pressed, only when File->Quit
+bool KHangMan::queryClose()
 {
-    Prefs::setShowCharToolbar( specialCharToolbar->isVisible() );
-    m_recent->saveEntries(KConfigGroup(config, "KHangManRecent"));
-    Prefs::self()->writeConfig();
-    qApp->closeAllWindows();
+    Prefs::setShowCharToolbar( m_specialCharToolbar->isVisible() );
+    m_recent->saveEntries(KConfigGroup(m_config, "KHangManRecent"));
+    Prefs::self()->save();
+    return KMainWindow::queryClose();
 }
 
 void KHangMan::slotChangeLevel(int index)
 {
     QMap<QString, QString>::const_iterator currentLevel = m_titleLevels.constBegin() + index;
-    changeStatusbar(currentLevel.key(), IDS_LEVEL);
+    m_levelLabel->setText(currentLevel.key());
     Prefs::setCurrentLevel(index);
     Prefs::setLevelFile(currentLevel.value());
-    Prefs::self()->writeConfig();
+    Prefs::self()->save();
     m_view->readFile();
     m_view->newGame();
 }
@@ -196,9 +192,9 @@ void KHangMan::slotChangeLevel(int index)
 void KHangMan::slotChangeLanguage(int index)
 {
     //good when no in English
-    kDebug() << "Change to " << m_languages[m_languageNames.indexOf(m_languageNames[index])];
+    qDebug() << "Change to " << m_languages[m_languageNames.indexOf(m_languageNames[index])];
     Prefs::setSelectedLanguage(m_languages[m_languageNames.indexOf(m_languageNames[index])]);
-    Prefs::self()->writeConfig();
+    Prefs::self()->save();
     loadLevels();
     loadLangToolBar();
     setAccent();
@@ -206,12 +202,12 @@ void KHangMan::slotChangeLanguage(int index)
     m_view->newGame();
 }
 
-void KHangMan::slotChangeMode(int index)
+void KHangMan::slotChangeTheme(int index)
 {
     Prefs::setMode(index);
-    Prefs::self()->writeConfig();
+    Prefs::self()->save();
     //m_view->setTheme(KHMThemeFactory::instance()->buildTheme(index));
-    m_view->setTheme(khm_factory.buildTheme(index));
+    m_view->setTheme(m_khmfactory.buildTheme(index));
 }
 
 
@@ -225,7 +221,7 @@ void KHangMan::setLanguages()
 
     //the program scans in khangman/data/ to see what languages data is found
     m_languages = SharedKvtmlFiles::languages();
-    kDebug() << "Languages " << m_languages;
+    qDebug() << "Languages " << m_languages;
     if (m_languages.isEmpty()) {
         qApp->closeAllWindows();
     }
@@ -233,22 +229,17 @@ void KHangMan::setLanguages()
 
     // Write the present languages in config so they cannot be downloaded.
     // FIXME: use pre-seeding here
-    KConfigGroup cg( KGlobal::config() ,"KNewStuff2");
+    KConfigGroup cg( KSharedConfig::openConfig() ,"KNewStuff2");
     for (int i=0;  i<m_languages.count(); ++i) {
         //QString tmp = cg.readEntry(m_languages[i]);
        // if (!tmp)
         cg.writeEntry(m_languages[i], QDate::currentDate().toString(Qt::ISODate));
     }
     cg.config()->sync();
-    
-    // We look in $KDEDIR/share/locale/all_languages from
-    // kdelibs/kdecore/all_languages to find the name of the country
-    // corresponding to the code and the language the user set.
 
-    KConfig entry(KStandardDirs::locate("locale", "all_languages"));
     for (int i = 0; i < m_languages.size(); ++i) {
-        KConfigGroup group = entry.group(m_languages[i]);
-        QString languageName = group.readEntry("Name");
+        QLocale locale(m_languages[i]);
+        QString languageName = locale.nativeLanguageName();
         if (languageName.isEmpty()) {
             languageName = i18nc("@item:inlistbox no language for that locale","None");
         }
@@ -263,16 +254,12 @@ void KHangMan::loadSettings()
     if (m_languages.isEmpty()){
         qApp->closeAllWindows();
     }
-    selectedLanguage = Prefs::selectedLanguage();
-    if (!m_languages.contains(selectedLanguage)) {
-        selectedLanguage = "en";
-    }
     // Show/hide characters toolbar
     if (Prefs::showCharToolbar()) {
-        specialCharToolbar->show();
+        m_specialCharToolbar->show();
     }
     else {
-        specialCharToolbar->hide();
+        m_specialCharToolbar->hide();
     }
     setMessages();
 }
@@ -286,22 +273,45 @@ void KHangMan::setLevel()
     m_view->readFile();
 }
 
-void KHangMan::loadLevels()
+void KHangMan::show()
+{
+    if (loadLevels()) { // kvtml files have been found
+        if (m_khmfactory.getQty() > 0) {  // themes present
+            slotChangeTheme(Prefs::mode());
+        } else { // themes not present
+            QMessageBox::information(this, i18n("Error"), i18n("No theme files found."));
+            close();
+            return;
+        }
+        //call show() of base class KXmlGuiWindow
+        KXmlGuiWindow::show();
+        m_view->newGame();
+    } else { // no kvtml files present
+        QMessageBox::information(this, i18n("Error"), i18n("No kvtml files found."));
+        close();
+    }
+}
+
+
+
+bool KHangMan::loadLevels()
 {
     //build the Level combobox menu dynamically depending of the data for each language
     m_titleLevels.clear();
     QStringList levelFilenames = SharedKvtmlFiles::fileNames(Prefs::selectedLanguage());
     QStringList titles = SharedKvtmlFiles::titles(Prefs::selectedLanguage());
-    
+
     if (levelFilenames.size() == 0) {
         Prefs::setSelectedLanguage("en");
-        Prefs::self()->writeConfig();
+        Prefs::self()->save();
         levelFilenames = SharedKvtmlFiles::fileNames(Prefs::selectedLanguage());
         titles = SharedKvtmlFiles::titles(Prefs::selectedLanguage());
     }
 
     if (levelFilenames.isEmpty()){
-        QApplication::instance()->quit();
+        // no kvtml files found
+        qDebug() << "levelFilenames.isEmpty() true";
+        return false;
     }
     m_languageAction->setCurrentItem(m_languages.indexOf(Prefs::selectedLanguage()));
 
@@ -314,7 +324,7 @@ void KHangMan::loadLevels()
         Prefs::setLevelFile(m_titleLevels.constBegin().value());
         Prefs::setCurrentLevel(0);
         m_currentLevel = 0;
-        Prefs::self()->writeConfig();
+        Prefs::self()->save();
     }
 
     // don't run off the end of the list
@@ -328,7 +338,9 @@ void KHangMan::loadLevels()
 
     setLevel();
     QMap<QString, QString>::const_iterator currentLevel = m_titleLevels.constBegin() + m_currentLevel;
-    changeStatusbar(currentLevel.key(), IDS_LEVEL);
+    m_levelLabel->setText(currentLevel.key());
+
+    return true;
 }
 
 
@@ -359,7 +371,7 @@ void KHangMan::optionsPreferences()
 
     connect(dialog, SIGNAL(settingsChanged(QString)), this, SLOT(updateSettings()));
     dialog->setAttribute( Qt::WA_DeleteOnClose );
-    dialog->setHelp(QString(),"khangman");
+    //dialog->setHelp(QString(),"khangman");
     dialog->show();
 }
 
@@ -370,7 +382,7 @@ void KHangMan::updateSettings()
     if (Prefs::selectedLanguage() == "de") {
         loadLangToolBar();
     }
-    hintAct->setChecked(Prefs::hint());
+    m_hintAct->setChecked(Prefs::hint());
 
     setMessages();
     //m_view->newGame();
@@ -398,27 +410,28 @@ void KHangMan::loadLangToolBar()
     QString lang = Prefs::selectedLanguage();
     if (lang.isEmpty())
         return;
-    m_specialChars = LangUtils::hasSpecialChars(lang);
+    bool hasSpecialChars = LangUtils::hasSpecialChars(lang);
 
-    if (specialCharToolbar->isVisible() && m_specialChars) {
+    // If the setting changed, save showToolbar setting
+    if (m_specialCharToolbar->isVisible() && hasSpecialChars) {
         Prefs::setShowCharToolbar(true);
-        Prefs::self()->writeConfig();
+        Prefs::self()->save();
     }
 
-    specialCharToolbar->clear();
+    m_specialCharToolbar->clear();
 
     m_allData.clear();
-    if (m_specialChars) {
+    if (hasSpecialChars) {
         QString langFileName=QString("khangman/%1.txt").arg(lang);
         QFile langFile;
-        langFile.setFileName(KStandardDirs::locate("data", langFileName));
+        langFile.setFileName(QStandardPaths::locate(QStandardPaths::GenericDataLocation, langFileName));
 
         // Let's look in local KDEHOME dir then KNS installs each .txt
         // in kvtml/<lang> as it installs everything at the same place
         if (!langFile.exists()) {
             langFileName = QString("kvtml/%1/%1.txt").arg(lang);
-            langFile.setFileName(KStandardDirs::locate("data", langFileName));
-            kDebug() << langFileName;
+            langFile.setFileName(QStandardPaths::locate(QStandardPaths::GenericDataLocation, langFileName));
+            qDebug() << langFileName;
         }
 
         if (!langFile.exists()) {
@@ -440,25 +453,16 @@ void KHangMan::loadLangToolBar()
 
         for (int i=0; i<m_allData.count(); ++i) {
             if (!m_allData.at(i).isEmpty()) {
-                QAction *act = specialCharToolbar->addAction(m_allData.at(i));
+                QAction *act = m_specialCharToolbar->addAction(m_allData.at(i));
                 act->setIcon(charIcon(m_allData.at(i).at(0)));
                 // used to carry the id
                 act->setData(i);
                 connect(act, SIGNAL(triggered(bool)), this, SLOT(slotPasteChar()));
             }
         }
-    }
-
-    if (Prefs::showCharToolbar()) {
-        specialCharToolbar->show();
-    }
-    else {
-        specialCharToolbar->hide();
-    }
-
-    // Hide toolbar for special characters if the language doesn't have them.
-    if (!m_specialChars) {
-        specialCharToolbar->hide();
+        m_specialCharToolbar->show();
+    } else {
+        m_specialCharToolbar->hide();
     }
 }
 
@@ -515,35 +519,39 @@ QIcon KHangMan::charIcon(const QChar & c) const
 
 void KHangMan::setAccent()
 {
-    kDebug() << "in slot accent  ";
+    qDebug() << "in slot accent  ";
     m_view->setAccentedLetters(LangUtils::hasAccentedLetters(Prefs::selectedLanguage()));
 }
 
+void KHangMan::setHintEnabled(bool enable)
+{
+    m_hintAct->setEnabled(enable);
+}
 
 void KHangMan::setMessages()
 {
     // Tell the user about accented characters
     if (m_view->accentedLetters() && Prefs::accentedLetters()) {
-        changeStatusbar(i18n("Type accented letters"), IDS_ACCENTS);
+        m_accentsLabel->setText(i18n("Type accented letters"));
     }
     else {
-        changeStatusbar("", IDS_ACCENTS);
+        m_accentsLabel->setText("");
     }
 }
 
-void KHangMan::loadFile(const KUrl & url)
+void KHangMan::loadFile(const QUrl &url)
 {
     if ( url.isValid() )  {
         if(url.isLocalFile())
             Prefs::setLevelFile(url.toLocalFile());
         else Prefs::setLevelFile(url.path());
-        
-       changeStatusbar(url.path().section('/', -1), IDS_LEVEL);
-       
+
+        m_levelLabel->setText(url.path().section('/', -1));
+
         m_recent->addUrl(url);
-        m_recent->saveEntries(KConfigGroup(config, "KHangManRecent"));
-        Prefs::self()->writeConfig();
-                
+        m_recent->saveEntries(KConfigGroup(m_config, "KHangManRecent"));
+        Prefs::self()->save();
+
         m_view->readFile();
         m_view->newGame();
     }
@@ -551,26 +559,38 @@ void KHangMan::loadFile(const KUrl & url)
 
 void KHangMan::slotNewGame()
 {
-    m_view->lossCount++;
-    statusBar()->changeItem(i18n("Losses: %1", m_view->lossCount), IDS_LOSSES);
-    m_view->newGame();
+    m_view->newGame(true);
+    m_hintAct->setChecked(Prefs::hint());
 }
 
-void KHangMan::slotOpenRecent(const KUrl & url)
+void KHangMan::slotOpenRecent(const QUrl &url)
 {
-    loadFile(url); 
+    loadFile(url);
 }
 
 void KHangMan::slotFileOpen()
 {
-    KUrl url = KFileDialog::getOpenUrl(QString(), KEduVocDocument::pattern(KEduVocDocument::Reading), this, i18n("Open Vocabulary Document"));
+    QUrl url = QFileDialog::getOpenFileUrl(this,
+                                           i18n("Open Vocabulary Document"),
+                                           QString(),
+                                           KEduVocDocument::pattern(KEduVocDocument::Reading));
     loadFile(url);
 }
 
+void KHangMan::slotSetWins(int wins)
+{
+    m_winsLabel->setText(i18n("Wins: %1", wins));
+}
+
+void KHangMan::slotSetLosses(int losses)
+{
+    m_lossesLabel->setText(i18n("Losses: %1", losses));
+}
+
 void KHangMan::slotSetHint(bool hint)
 {
     Prefs::setHint(hint);
-    Prefs::self()->writeConfig();
+    Prefs::self()->save();
     m_view->update();
 }
 
